/*
 * Segment 2 (KEx), SWE LMS-TIC3
 * Copyright (C) 2016 BitCtrl Systems GmbH 
 * 
 * This library is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Lesser General Public License as published by the Free
 * Software Foundation; either version 2.1 of the License, or (at your option)
 * any later version.
 *
 * This library is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public License for more
 * details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this library; if not, write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA.
 *
 * Contact Information:
 * BitCtrl Systems GmbH
 * Weißenfelser Straße 67
 * 04229 Leipzig
 * Phone: +49 341-490670
 * mailto: info@bitctrl.de
 */

package de.bsvrz.kex.lmstic3.vew;

import java.util.concurrent.TimeUnit;

import org.w3c.dom.Document;
import org.w3c.dom.Element;

import de.bsvrz.dav.daf.main.ClientDavInterface;
import de.bsvrz.dav.daf.main.ClientReceiverInterface;
import de.bsvrz.dav.daf.main.ClientSenderInterface;
import de.bsvrz.dav.daf.main.Data;
import de.bsvrz.dav.daf.main.Data.Array;
import de.bsvrz.dav.daf.main.Data.TextValue;
import de.bsvrz.dav.daf.main.DataDescription;
import de.bsvrz.dav.daf.main.DataNotSubscribedException;
import de.bsvrz.dav.daf.main.OneSubscriptionPerSendData;
import de.bsvrz.dav.daf.main.ReceiveOptions;
import de.bsvrz.dav.daf.main.ReceiverRole;
import de.bsvrz.dav.daf.main.ResultData;
import de.bsvrz.dav.daf.main.SendSubscriptionNotConfirmed;
import de.bsvrz.dav.daf.main.SenderRole;
import de.bsvrz.dav.daf.main.config.DataModel;
import de.bsvrz.dav.daf.main.config.SystemObject;
import de.bsvrz.kex.lmstic3.vew.daten.tic.TIC3Exception;
import de.bsvrz.kex.lmstic3.vew.daten.tic.TIC3MeldungsDaten;
import de.bsvrz.sys.funclib.debug.Debug;

/**
 * SWE-Interne Repr&auml;sentation einer TMC-Meldung.
 *
 * @author BitCtrl Systems GmbH, Gieseler
 * @version $Id: TMCMeldung.java 64047 2016-10-27 15:42:01Z gieseler $
 */

public class TMCMeldung implements ClientSenderInterface, ClientReceiverInterface {

	/**
	 * DebugLogger f&uuml;r Debug-Ausgaben.
	 */
	private static final Debug LOGGER = Debug.getLogger();

	/**
	 * Typ der Meldungen.
	 */
	public static final String TYP = "typ.tmcVerkehrsMeldung";

	public static final String ATG = "atg.tmcVerkehrsMeldung";

	private static final String ASP_EMPFANG = "asp.tmcEmpfangen";
	private static final String ASP_SENDEN = "asp.tmcSenden";
	private static final String ASP_VERSENDET = "asp.tmcVersendet";

	private static final String PID_PREFIX = "tmc.lms.";

	/**
	 * der interne Status der Meldung.
	 */
	private MeldungsStatus status = MeldungsStatus.VERSENDET;

	/**
	 * Das zugeh&ouml;rige Systemobjekt.
	 */
	private final SystemObject systemObject;

	/**
	 * Flag, ob die Datenanmeldung beim DaV erfolgt ist.
	 */
	private boolean istAngemeldet;

	/**
	 * Synchronisationsobjekt f&uuml;r die Datenanmeldung.
	 */
	private final Object anmeldungsLock = new Object();

	/** Wartezeit auf erfolgreiche Anmeldebest&auml;tigung. */
	private final long warteZeitAnmeldung = TimeUnit.SECONDS.toMillis(20);

	/** Zeitpunkt des letzten Versandes. */
	private long letzterVersand;

	// /** die Daten der Meldung. */
	// private TIC3MeldungsDaten meldungsDaten;

	/** Datenspezifikation f&uuml;r die empfangenen Daten. */
	private DavSendeAnmeldung dataEmpfangen;

	/** Datenspezifikation f&uuml;r die empfangenen Daten als Parameter. */
	// private DavSendeAnmeldung dataParameter;

	/** Datenspezifikation f&uuml;r die zu versendenden Daten. */
	private DavSendeAnmeldung dataVersand;

	/** Datenspezifikation f&uuml;r die 'versendet' Daten. */
	private DavSendeAnmeldung dataVersendet;

	/** Flag, ob f&uuml;r die Meldung Daten vom DaV empfangen wurden. */
	private boolean davDatenOK;

	/** Flag, ob die Meldung versendet wurde. */
	private boolean versendet;

	private Data davData;

	private TIC3MeldungsDaten tic3Daten;

	/** Kopie der DAV-Daten zum Versenden. */
	private Data davDatenVersenden;

	/**
	 * Erzeugt eine neue TMC-Meldung aus einem Systemobjekt.
	 *
	 * @param object
	 *            Systemobjekt
	 */
	public TMCMeldung(final SystemObject object) {
		systemObject = object;
		tic3Daten = new TIC3MeldungsDaten();
	}

	/**
	 * Erzeugt eine neue TMC-Meldung aus einem Systemobjekt und Meldungsdaten.
	 *
	 * @param object
	 *            Systemobjekt
	 * @param daten
	 *            Meldungsdaten
	 */
	public TMCMeldung(final SystemObject object, final Data daten) {
		this(object);
		davData = daten;
	}

	public Data getDavData() {
		return davDatenVersenden;
	}

	/**
	 * Aktualisiert die Daten der Meldung.
	 *
	 * @param dav
	 *            {@link ClientDavInterface}
	 * @param data
	 *            DaV-Daten
	 * @throws TIC3Exception
	 *             bei Ausnahmen
	 */
	public void aktualisieren(final ClientDavInterface dav, final Data data) throws TIC3Exception {
		if (data == null) {
			throw new TIC3Exception("Aktualisierung '" + systemObject + "': Keine Daten");
		}

		final ResultData meldungsdaten = new ResultData(systemObject, dataEmpfangen, System.currentTimeMillis(), data);
		try {
			dav.sendData(meldungsdaten);
		} catch (final DataNotSubscribedException | SendSubscriptionNotConfirmed e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
	}

	/**
	 * Aktualisiert die Daten der Meldung.
	 *
	 * @param data
	 *            DaV-Daten
	 * @throws TIC3Exception
	 *             bei Ausnahmen
	 */
	public void aktualisieren(final Data data) throws TIC3Exception {
		if (data == null) {
			throw new TIC3Exception("Aktualisierung '" + systemObject + "': Keine Daten");
		}

		davData = data;
	}

	/**
	 * Datenverteileranmeldung f&uuml;r empfangene Meldungen.
	 *
	 * @param dav
	 *            die Datenverteilerverbindung
	 * @throws TIC3Exception
	 *             bei Ausnahmen
	 */
	public void datenAktualisierenDav(final ClientDavInterface dav) throws TIC3Exception {
		final DataModel model = dav.getDataModel();

		Data data = null;

		if (data == null) {
			data = dav
					.getData(systemObject,
							new DataDescription(model.getAttributeGroup(ATG), model.getAspect(ASP_SENDEN)), 0)
					.getData();
		}

		if (data == null) {
			throw new TIC3Exception("keine aktuellen Daten unter dem Aspekt '" + ASP_SENDEN + "'");
		}

		aktualisieren(data);
	}

	/**
	 * Datenverteileranmeldung f&uuml;r empfangene Meldungen.
	 *
	 * @param dav
	 *            die Datenverteilerverbindung
	 * @throws TIC3Exception
	 *             bei Ausnahmen
	 */
	public void anmeldenDavEmpfang(final ClientDavInterface dav) throws TIC3Exception {

		if (istAngemeldet) {
			return;
		}

		final DataModel model = dav.getDataModel();

		dataEmpfangen = new DavSendeAnmeldung(model.getAttributeGroup(ATG), model.getAspect(ASP_EMPFANG));

		try {
			TMCMeldung.LOGGER.fine("beginne Datenanmeldung Empfangsdaten: " + this);

			synchronized (anmeldungsLock) {
				dav.subscribeSender(this, systemObject, dataEmpfangen, SenderRole.source());

				// warte auf Anmeldebestätigung
				TMCMeldung.LOGGER.fine("beginne Wartezeit auf Bestätigung der Datenanmeldung: " + this);

				try {
					anmeldungsLock.wait(warteZeitAnmeldung);
				} catch (final InterruptedException e) {
					// wird nicht behandelt
				}
			}

			TMCMeldung.LOGGER.fine("Ende Warten auf Bestätigung der Datenanmeldung: " + this);

			if (!istAngemeldet) {
				throw new TIC3Exception("Die Meldung " + systemObject.getPid()
						+ " konnte nicht erfolgreich am DaV angemeldet werden" + " (es erfolgte innerhalb von "
						+ (warteZeitAnmeldung / 1000) + "s keine Rückmeldung über die Sendeanmeldung)");
			}

		} catch (final OneSubscriptionPerSendData e) {
			throw new TIC3Exception("Fehler bei DAV Anmeldung: Objekt: " + systemObject + " (" + e.getMessage() + ")");
		}
	}

	/**
	 * Datenverteilerabmeldung f&uuml;r empfangene Meldungen.
	 *
	 * @param dav
	 *            die Datenverteilerverbindung
	 * @throws TIC3Exception
	 *             bei Ausnahmen
	 */
	public void abmeldenDavEmpfang(final ClientDavInterface dav) throws TIC3Exception {
		dav.unsubscribeSender(this, systemObject, dataEmpfangen);
	}

	public static String bildePid(final String id) {
		return PID_PREFIX + id;
	}

	/**
	 * Datenverteileranmeldung f&uuml;r zu versendende Meldungen.
	 *
	 * @param dav
	 *            die Datenverteilerverbindung
	 * @throws TIC3Exception
	 *             bei Ausnahmen
	 */
	public void anmeldenDavVersand(final ClientDavInterface dav) throws TIC3Exception {

		if (istAngemeldet) {
			return;
		}

		final DataModel model = dav.getDataModel();

		dataVersand = new DavSendeAnmeldung(model.getAttributeGroup(ATG), model.getAspect(ASP_SENDEN));

		dataVersendet = new DavSendeAnmeldung(model.getAttributeGroup(ATG), model.getAspect(ASP_VERSENDET));

		final SystemObject[] objekte = new SystemObject[] { systemObject };

		try {
			dav.subscribeSender(this, systemObject, dataVersendet, SenderRole.source());
		} catch (final OneSubscriptionPerSendData e) {
			throw new TIC3Exception(e.getMessage());
		}

		dav.subscribeReceiver(this, objekte, dataVersand, new ReceiveOptions(false), // keine
				// Deltas,
				// keine
				// nachgelieferten
				// Daten
				ReceiverRole.receiver());
	}

	/**
	 * Datenverteilerabmeldung f&uuml;r zu versendende Meldungen.
	 *
	 * @param dav
	 *            die Datenverteilerverbindung
	 * @throws TIC3Exception
	 *             bei Ausnahmen
	 */
	public void abmeldenDavVersand(final ClientDavInterface dav) throws TIC3Exception {
		final SystemObject[] objekte = new SystemObject[] { systemObject };

		dav.unsubscribeSender(this, systemObject, dataVersendet);
		dav.unsubscribeReceiver(this, objekte, dataVersand);
	}

	@Override
	public void dataRequest(final SystemObject object, final DataDescription dataDescription, final byte state) {
		boolean anmeldungOK = false;

		TMCMeldung.LOGGER.finer("dataRequest: " + this + ", DataDescription: " + dataDescription);

		if (dataDescription == dataEmpfangen) {
			// die Daten werden auch gesendet, wenn kein Empfänger angemeldet
			// ist
			dataEmpfangen.setzeSendenOK(
					(state == ClientSenderInterface.START_SENDING) || (state == ClientSenderInterface.STOP_SENDING));
		}

		if (dataEmpfangen != null) {
			anmeldungOK = dataEmpfangen.sendenOK();
		}

		if (anmeldungOK && !istAngemeldet) {
			istAngemeldet = true;
			TMCMeldung.LOGGER
					.finer("dataRequest: " + this + " Lock-Freigabe: " + istAngemeldet + ", " + anmeldungsLock);
			if (anmeldungsLock != null) {
				synchronized (anmeldungsLock) {
					anmeldungsLock.notify();
				}
			}
		}
	}

	/**
	 * Vervollst&auml;ndigt die Daten der Meldung.
	 *
	 * <p>
	 * DataProducer: aus TMC-Meldung oder wenn nicht gesetzt aus Parameter
	 * atg.tmcKommunikationsParameter.EigeneID
	 * </p>
	 * <p>
	 * DataIdentifier: inkrementierte laufende Nummer aus
	 * atg.lmsKommunikationsParameter.LaufendeNummer
	 * </p>
	 * 
	 * @param parameter
	 *            LMS-Parameter
	 */
	public void datenvervollstaendigen(final LMSParameter parameter) {
		final Data item = davDatenVersenden.getItem("TMCID");
		final TextValue kennung = item.getTextValue("Datenkennung");
		if (kennung.getText().isEmpty()) {
			kennung.setText("" + parameter.getLaufendeNummerMeldung());
		}

		final TextValue prod = item.getTextValue("Datenproduzent");
		if (prod.getText().isEmpty()) {
			prod.setText(parameter.getEigeneID());
		}
	}

	/**
	 * Erzeugt das XML-Element f&uuml;r die Meldung.
	 *
	 * @param doc
	 *            Dokument
	 * @param root
	 *            Wurzelelement
	 * @throws TIC3Exception
	 *             bei Ausnahmen
	 */
	public void erzeugeDocumentElement(final Document doc, final Element root) throws TIC3Exception {
		tic3Daten.erzeugeDocumentElement(doc, root);
	}

	/**
	 * Gibt den Zeitpunkt des letzten Versandes der Meldung zur&uuml;ck.
	 *
	 * @return Zeitpunkt des letzten Versandes
	 */
	public long getLetzterVersand() {
		return letzterVersand;
	}

	/**
	 * Gibt die PID des zugeh&ouml;rigen Systemobjektes zur&uuml;ck.
	 *
	 * @return PID
	 */
	public String getPID() {
		return systemObject.getPid();
	}

	/**
	 * Gibt den internen Status der Meldung zur&uuml;ck.
	 *
	 * @return Status
	 */
	public MeldungsStatus getStatus() {
		return status;
	}

	/**
	 * Gibt das zugeh&ouml;rige Systemobjekt zur&uuml;ck.
	 *
	 * @return zugeh&ouml;riges Systemobjekt
	 */
	public SystemObject getSystemObject() {
		return systemObject;
	}

	/**
	 * Test, ob f&uuml;r die Meldung Daten vom DaV empfangen wurden.
	 *
	 * @return true, wenn Daten vom DaV empfangen wurden, sonst false
	 */
	public boolean isDavDatenOK() {
		return davDatenOK;
	}

	@Override
	public boolean isRequestSupported(final SystemObject object, final DataDescription dataDescription) {
		return true;
	}

	/**
	 * Test, ob die Meldung versendet wurde.
	 *
	 * @return true, wenn versendet, sonst false
	 */
	public boolean isVersendet() {
		return versendet;
	}

	/**
	 * Publiziert die Daten zum DaV, dass die Meldung versendet wurde.
	 *
	 * @param dav
	 *            Datenverteilerverbindung
	 */
	public void publiziereVersendet(final ClientDavInterface dav) {
		try {
			final Data data;

			// Benutze Kopie der Daten zum Versand, wenn vorhanden
			if (davDatenVersenden != null) {
				data = davDatenVersenden;
			} else {
				data = dav.createData(dataVersendet.getAttributeGroup());
			}

			final ResultData meldungsdaten = new ResultData(systemObject, dataVersendet, System.currentTimeMillis(),
					data);

			TMCMeldung.LOGGER.finest("Sende Daten (versendet) zum DAV: " + meldungsdaten);
			dav.sendData(meldungsdaten);
		} catch (final DataNotSubscribedException e) {
			TMCMeldung.LOGGER.error(e.getMessage());
		} catch (final SendSubscriptionNotConfirmed e) {
			TMCMeldung.LOGGER.error(e.getMessage());
		}
	}

	/**
	 * Setzt den Zeitpunlt des letzten Versandes der Meldung.
	 *
	 * @param versand
	 *            Zeitpunkt letzter Versand
	 */
	public void setLetzterVersand(final long versand) {
		letzterVersand = versand;
	}

	/**
	 * Setzt den internen Status der Meldung.
	 *
	 * @param status
	 *            Status
	 */
	public void setStatus(final MeldungsStatus status) {
		this.status = status;
	}

	/**
	 * Setzt das Flag, ob die Meldung versendet wurde.
	 *
	 * @param versendet
	 *            Meldung versendet
	 */
	public void setVersendet(final boolean versendet) {
		this.versendet = versendet;
	}

	@Override
	public String toString() {
		return getPID();
	}

	@Override
	public void update(final ResultData[] results) {
		for (final ResultData datensatz : results) {
			if (datensatz.hasData()) {
				TMCMeldung.LOGGER.finer("Datensatz erhalten für ", datensatz.getObject());
				if (datensatz.getDataDescription().equals(dataVersand)) {
					TMCMeldung.LOGGER.finest("Datensatz: ", datensatz);
					try {
						// Kopie der empfangenen Daten zum Versand anlegen, da
						// die interne Repräsentation nicht alle Daten enthält
						// (ATG wurde von Dambach erweitert)
						davDatenVersenden = datensatz.getData().createModifiableCopy();

						aktualisieren(datensatz.getData());
						davDatenOK = true;
						setStatus(MeldungsStatus.AKTUALISIERT);
					} catch (final TIC3Exception e) {
						TMCMeldung.LOGGER.error(e.getMessage());
					}
				}
			}
		}
	}

	public boolean istZeitGueltig() {
		final Array array = davData.getItem("TMCDaten").getItem("VerwaltungsInformationen").getArray("EndeZeit");
		if (array.getLength() > 0) {
			final long millis = array.getItem(0).asTimeValue().getMillis();
			return millis > System.currentTimeMillis();
		}

		return true;
	}

	public void setData(final Data createData) {
		davData = createData;
	}

	public void publiziereDav(final ClientDavInterface dav) throws TIC3Exception {
		aktualisieren(dav, davData);
	}

	/**
	 * Bildet die TicId.
	 * 
	 * @return TicId
	 */
	public String bildeTICId() {
		final Data item = davData.getItem("TMCID");
		return item.getTextValue("Datenproduzent").getText() + "/" + item.getTextValue("Datenkennung").getText();
	}

}
