/*
 * Segment 2 (KEx), SWE LMS-TIC3
 * Copyright (C) 2016 BitCtrl Systems GmbH 
 * 
 * This library is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Lesser General Public License as published by the Free
 * Software Foundation; either version 2.1 of the License, or (at your option)
 * any later version.
 *
 * This library is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public License for more
 * details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this library; if not, write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA.
 *
 * Contact Information:
 * BitCtrl Systems GmbH
 * Weißenfelser Straße 67
 * 04229 Leipzig
 * Phone: +49 341-490670
 * mailto: info@bitctrl.de
 */

package de.bsvrz.kex.lmstic3.vew.daten.tic.elemente;

import java.util.ArrayList;
import java.util.List;

import org.w3c.dom.Node;
import org.w3c.dom.NodeList;

import de.bsvrz.dav.daf.main.Data;
import de.bsvrz.kex.lmstic3.vew.LMSParameter;
import de.bsvrz.kex.lmstic3.vew.daten.tic.AbstractTIC3InfoElement;
import de.bsvrz.kex.lmstic3.vew.daten.tic.TIC3Boolean;
import de.bsvrz.kex.lmstic3.vew.daten.tic.TIC3Exception;

/**
 * TIC3-Ergebniserzeugung.
 * <p>
 * Entspricht TIC3-Typ 'CreateResultType'.
 * </p>
 *
 * @author BitCtrl Systems GmbH, Gieseler
 * @version $Id: TIC3ErgebnisErzeugung.java 63650 2016-09-22 17:11:32Z gieseler $
 */
public class TIC3ErgebnisErzeugung extends AbstractTIC3InfoElement {
	/**
	 * TIC3 Info XML Element: CreateResult (CRE), Settings used to create the
	 * data contained in this object.
	 */
	public static final String XML_ELEMENT_ERGEBNIS = "CreateResult";

	/**
	 * Automatisch aktualisieren.
	 * <p>
	 * TIC3 Info XML Element: UpdateAutomatically (UAT), Specifies whether or
	 * not the source data is used to automatically update the data contained in
	 * this object.
	 * </p>
	 */
	private TIC3Boolean update = new TIC3Boolean("UpdateAutomatically", "AutomatischAktualisieren");

	/**
	 * Regeln.
	 * <p>
	 * TIC3 Info XML Element: Rule (RUL), A list of rules to be used when this
	 * source data is used to create the data contained in this object.
	 * </p>
	 */
	private static final String DAV_ATTR_REGELN = "ErgebnisErzeugungRegeln";
	private final List<TIC3ErgebnisErzeugungRegel> regeln = new ArrayList<>();

	/**
	 * Erzeugt ein neues Objekt.
	 *
	 * @param xmlName
	 *            Name des XML-ELementes
	 * @param davName
	 *            Name des DAV-Attributes
	 */
	public TIC3ErgebnisErzeugung(final String xmlName, final String davName) {
		super(xmlName, davName);
		addKindElement(update);
	}

	@Override
	public void einlesenXML(final Node node) throws TIC3Exception {
		super.einlesenXML(node);

		final NodeList nl = node.getChildNodes();

		for (int i = 0; i < nl.getLength(); i++) {
			final Node n = nl.item(i);

			// Regeln
			if (n.getNodeName().equals(TIC3ErgebnisErzeugungRegel.XML_ELEMENT_REGEL)) {
				final TIC3ErgebnisErzeugungRegel regel = new TIC3ErgebnisErzeugungRegel();
				regel.einlesenXML(n);
				regeln.add(regel);
			}
		}
	}

	@Override
	public void publizieren(final Data davdata, final LMSParameter parameter) throws TIC3Exception {
		super.publizieren(davdata, parameter);

		final Data ddata = getLocalDavData(davdata);

		// Regeln
		if (regeln.size() > 0) {
			final Data.Array array = ddata.getArray(DAV_ATTR_REGELN);
			array.setLength(regeln.size());
			for (int i = 0; i < regeln.size(); i++) {
				regeln.get(i).publizieren(array.getItem(i), parameter);
			}
		}
	}

	@Override
	public String toString() {
		return this.getClass().getSimpleName();
	}
}
