/*
 * Segment 2 (KEx), SWE LMS-TIC3
 * Copyright (C) 2016 BitCtrl Systems GmbH 
 * 
 * This library is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Lesser General Public License as published by the Free
 * Software Foundation; either version 2.1 of the License, or (at your option)
 * any later version.
 *
 * This library is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public License for more
 * details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this library; if not, write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA.
 *
 * Contact Information:
 * BitCtrl Systems GmbH
 * Weißenfelser Straße 67
 * 04229 Leipzig
 * Phone: +49 341-490670
 * mailto: info@bitctrl.de
 */


package de.bsvrz.kex.lmstic3.vew.daten.tic.elemente;

import org.w3c.dom.Node;

import de.bsvrz.dav.daf.main.Data;
import de.bsvrz.kex.lmstic3.vew.daten.tic.TIC3Exception;
import de.bsvrz.kex.lmstic3.vew.daten.tic.TMCInteger;

/**
 * TIC3-Himmelsrichtung.
 * <p>Entspricht TIC3 'CompassDirectionEnum'.</p>
 *
 * @author BitCtrl Systems GmbH, Gieseler
 * @version $Id: TIC3Himmelsrichtung.java 63650 2016-09-22 17:11:32Z gieseler $     
 */

public class TIC3Himmelsrichtung extends TMCInteger {
	
	/**
	 * TTIC3 Info XML Element: CompassDirection (CPS), The name corresponding to a compass direction such as "North", "South", "East", or "West".
	 */
	public static final String XML_ELEMENT_COMPASSDIRECTION = "CompassDirection";

	
	/**
	 * Enumerator TIC3-Himmelsrichtung.
	 */
	public enum TIC3HimmelsrichtungEnum {
		
		/** NichtVersorgt. */
		NichtVersorgt("NichtVersorgt", -1),

		/** North. */
		Nord("Nord", 1),

		/** Süd. */
		Sued("Süd", 2),

		/** Ost. */
		Ost("Ost", 3), 
		
		/** West. */
		West("West", 4), 
		
		/** Nordwest. */
		Nordwest("Nordwest", 5), 
		
		/** Nordost. */
		Nordost("Nordost", 6), 
		
		/** Südwest. */
		Suedwest("Südwest", 7),
		
		/** Südost. */
		Suedost("Südost", 8);
		
		/**
		 * Gibt eine passende Himmelsrichtung-Instanz zur&uuml;ck.
		 *
		 * @param id
		 *            numerischer Wert
		 * @return zum numerischen Wert passende Instanz
		 * @throws TIC3Exception
		 *             bei ung&uuml;ltigem Wert
		 */
		public static TIC3HimmelsrichtungEnum getInstance(final int id) throws TIC3Exception {
			switch (id) {
			case -1:
				return NichtVersorgt;

			case 1:
				return Nord;
			case 2:
				return Sued;
			case 3:
				return Ost;
			case 4:
				return West;
			case 5:
				return Nordwest;
			case 6:
				return Nordost;
			case 7:
				return Suedwest;
			case 8:
				return Suedost;
			default:	
				throw new TIC3Exception("Keine TIC3-Himmelsrichtung mit id=" + id + " definiert.");
			}
		}

		/**
		 * textueller Wert.
		 */
		private final String value;

		/**
		 * numerischer Wert.
		 */
		private int id;

		/**
		 * Privater Konstruktor.
		 *
		 * @param value
		 *            textueller Wert
		 * @param id
		 *            numerischer Wert
		 */
		TIC3HimmelsrichtungEnum(final String value, final int id) {
			this.value = value;
			this.id = id;
		}

		/**
		 * Gibt den numerischen Wert zur&uuml;ck.
		 *
		 * @return numerischer Wert der Himmelsrichtung
		 */
		public int getId() {
			return id;
		}

		/**
		 * Gibt den textuellen Wert zur&uuml;ck.
		 *
		 * @return textueller Wert der Himmelsrichtung
		 */
		public String getValue() {
			return value;
		}
	}

	/**
	 * Himmelsrichtung.
	 */
	private TIC3HimmelsrichtungEnum richtung = TIC3HimmelsrichtungEnum.NichtVersorgt;
	

	/**
	 * Erzeugt ein neues Objekt.
	 *
	 * @param xmlName
	 *            Name des XML-ELementes
	 * @param davName
	 *            Name des DAV-Attributes
	 */
	public TIC3Himmelsrichtung(final String xmlName, final String davName) {
		super(xmlName, davName);
	}

	@Override
	public void getDavDaten(final Data davdata) throws TIC3Exception {
		super.getDavDaten(davdata);
		richtung = TIC3HimmelsrichtungEnum.getInstance(getWert());
	}

	@Override
	public void getXmlDaten(final Node node) throws TIC3Exception {
		richtung = TIC3HimmelsrichtungEnum.getInstance(Integer.parseInt(node.getTextContent()));
	}

	@Override
	public Integer getWert() {
		return richtung.getId();
	}
}
