/*
 * Segment 2 (KEx), SWE LMS-TIC3
 * Copyright (C) 2016 BitCtrl Systems GmbH 
 * 
 * This library is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Lesser General Public License as published by the Free
 * Software Foundation; either version 2.1 of the License, or (at your option)
 * any later version.
 *
 * This library is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public License for more
 * details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this library; if not, write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA.
 *
 * Contact Information:
 * BitCtrl Systems GmbH
 * Weißenfelser Straße 67
 * 04229 Leipzig
 * Phone: +49 341-490670
 * mailto: info@bitctrl.de
 */

package de.bsvrz.kex.lmstic3.vew.daten.tic.elemente;

import java.util.ArrayList;
import java.util.List;

import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;

import de.bsvrz.dav.daf.main.Data;
import de.bsvrz.kex.lmstic3.vew.LMSParameter;
import de.bsvrz.kex.lmstic3.vew.daten.tic.AbstractTIC3InfoElement;
import de.bsvrz.kex.lmstic3.vew.daten.tic.TIC3Exception;
import de.bsvrz.kex.lmstic3.vew.daten.tic.TIC3InfoAttribut;

/**
 * TIC3-Lokationsinformationen.
 * 
 * <p>
 * TIC3 Info XML Element: Location (LOC), Information about a location specified
 * by one or more location referencing methods such as TIC3 location,
 * geographical position, or TMC location.
 * </p>
 *
 * @author BitCtrl Systems GmbH, Gieseler
 * @version $Id: TIC3Location.java 63650 2016-09-22 17:11:32Z gieseler $
 */
public class TIC3Location extends AbstractTIC3InfoElement {

	/**
	 * TIC3 Info XML Element: Location (LOC), Information about a location
	 * specified by one or more location referencing methods such as TIC3
	 * location, geographical position, or TMC location.
	 */
	public static final String XML_ELEMENT_LOCATIONLOC = "Location";

	/**
	 * TIC3-Lokationsinformationen mit WGS-Koordinaten.
	 * <p>
	 * TIC3 Info XML Element: Position (POS), The position as one or more
	 * geographical coordinates (WGS84).
	 * </p>
	 */
	/**
	 * TIC3 Info XML Element: Position (POS), The position as one or more
	 * geographical coordinates (WGS84).
	 */
	private static final String XML_ELEMENT_POSITIONLOC = "Position";

	/**
	 * DAV-Attribut.
	 */
	private static final String DAV_ATTR_POSITIONLOC = "TIC3KoordinatenLocation";
	private final TIC3ShapeType tic3KoordinatenLocation = new TIC3ShapeType(XML_ELEMENT_POSITIONLOC,
			DAV_ATTR_POSITIONLOC);

	/**
	 * Beschreibung.
	 * <p>
	 * Beschreibung aus Wert des 'description'-Attributes.
	 * </p>
	 */
	private TIC3InfoAttribut beschreibung = new TIC3InfoAttribut("description", "Beschreibung");

	private List<TIC3TMCLocation> tic3TMCLocationen = new ArrayList<>();

	private List<TIC3TICLocation> tic3TICLocationen = new ArrayList<>();

	/**
	 * Erzeugt ein neues Objekt.
	 *
	 * @param xmlName
	 *            Name des XML-ELementes
	 * @param davName
	 *            Name des DAV-Attributes
	 */
	public TIC3Location(final String xmlName, final String davName) {
		super(xmlName, davName);
		addAttribut(beschreibung);
		addKindElement(tic3KoordinatenLocation);
	}

	@Override
	public void einlesenXML(final Node node) throws TIC3Exception {
		super.einlesenXML(node);
		final NodeList nl = node.getChildNodes();

		for (int i = 0; i < nl.getLength(); i++) {
			final Node n = nl.item(i);
			if (n.getNodeName().equals(TIC3TMCLocation.XML_ELEMENT_TMCLOC)) {
				final TIC3TMCLocation loc = new TIC3TMCLocation();
				loc.einlesenXML(n);
				tic3TMCLocationen.add(loc);
			}

			if (n.getNodeName().equals(TIC3TICLocation.XML_ELEMENT_TICLOC)) {
				final TIC3TICLocation loc = new TIC3TICLocation();
				loc.einlesenXML(n);
				tic3TICLocationen.add(loc);
			}
		}
	}

	@Override
	public void publizieren(final Data davdata, final LMSParameter parameter) throws TIC3Exception {
		super.publizieren(davdata, parameter);

		final Data ddata = getLocalDavData(davdata);

		// TMC-Lokationen
		if (tic3TMCLocationen.size() > 0) {
			final Data.Array array = ddata.getArray(TIC3TMCLocation.DAV_ATTR_TMCLOC);

			array.setLength(tic3TMCLocationen.size());
			for (int i = 0; i < tic3TMCLocationen.size(); i++) {
				tic3TMCLocationen.get(i).publizieren(array.getItem(i), parameter);
			}
		}

		// TIC-Lokationen
		if (tic3TICLocationen.size() > 0) {
			final Data.Array array = ddata.getArray(TIC3TICLocation.DAV_ATTR_TICLOC);

			array.setLength(tic3TICLocationen.size());
			for (int i = 0; i < tic3TICLocationen.size(); i++) {
				tic3TICLocationen.get(i).publizieren(array.getItem(i), parameter);
			}
		}
	}

	@Override
	public Element erzeugeDocumentElement(final Document doc, final Element parent) throws TIC3Exception {
		final Element element = super.erzeugeDocumentElement(doc, parent);

		if (tic3TMCLocationen.size() > 0) {
			for (TIC3TMCLocation ev : tic3TMCLocationen) {
				ev.erzeugeDocumentElement(doc, element);
			}
		}

		return element;
	}

	public List<TIC3TMCLocation> getTic3TMCLocationen() {
		return tic3TMCLocationen;
	}

	public void setTic3TMCLocationen(final List<TIC3TMCLocation> locationen) {
		tic3TMCLocationen.addAll(locationen);
	}

	public List<TIC3TICLocation> getTic3TICLocationen() {
		return tic3TICLocationen;
	}

	public List<TIC3Koordinaten> getKoordinaten() {
		return tic3KoordinatenLocation.getKoordinaten();
	}

}
