/*
 * Segment 2 (KEx), SWE LMS-TIC3
 * Copyright (C) 2016 BitCtrl Systems GmbH 
 * 
 * This library is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Lesser General Public License as published by the Free
 * Software Foundation; either version 2.1 of the License, or (at your option)
 * any later version.
 *
 * This library is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public License for more
 * details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this library; if not, write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA.
 *
 * Contact Information:
 * BitCtrl Systems GmbH
 * Weißenfelser Straße 67
 * 04229 Leipzig
 * Phone: +49 341-490670
 * mailto: info@bitctrl.de
 */

package de.bsvrz.kex.lmstic3.vew.daten.tic.elemente;

import java.util.ArrayList;
import java.util.List;

import org.w3c.dom.Node;
import org.w3c.dom.NodeList;

import de.bsvrz.dav.daf.main.Data;
import de.bsvrz.kex.lmstic3.vew.LMSParameter;
import de.bsvrz.kex.lmstic3.vew.daten.tic.AbstractTIC3InfoElement;
import de.bsvrz.kex.lmstic3.vew.daten.tic.TIC3Boolean;
import de.bsvrz.kex.lmstic3.vew.daten.tic.TIC3Exception;
import de.bsvrz.kex.lmstic3.vew.daten.tic.TIC3InfoAttribut;
import de.bsvrz.kex.lmstic3.vew.daten.tic.TIC3OptionalerDoubleWert;

/**
 * TIC3-Umleitung.
 * <p>
 * Entspricht TIC3-Typ 'DetourRouteType'.
 * </p>
 *
 * @author BitCtrl Systems GmbH, Gieseler
 * @version $Id: TIC3Umleitung.java 63650 2016-09-22 17:11:32Z gieseler $
 */
public class TIC3Umleitung extends AbstractTIC3InfoElement {
	/**
	 * TIC3 Info XML Element: DiversionRoute (DRO), A list of diversion routes
	 * and the vehicle types to which the diversion applies.
	 */
	public static final String XML_ELEMENT_UMLEITUNG = "DiversionRoute";

	/**
	 * Name.
	 * <p>
	 * TIC3 Info XML Element: Name (NAM), The name of the diversion route, in
	 * one or more languages configured on the system. The name is used to
	 * easily identify this diversion route when later viewing or selecting this
	 * route in another window.
	 * </p>
	 */
	private TIC3Text name = new TIC3Text("Name", "Name");

	/**
	 * Beschreibung.
	 * <p>
	 * TIC3 Info XML Element: Description (DES), The description of the
	 * diversion route, in one or more languages configured on the system. The
	 * description is usually a summary containing enough detail for travellers
	 * to understand and decide if they will use this diversion route. The
	 * description becomes part of the traffic information provided.
	 * </p>
	 */
	private TIC3Text beschreibung = new TIC3Text("Description", "Beschreibung");

	/**
	 * Die Beschreibung ist in untersuchten Dateien etwas seltsam:
	 * <p>
	 * <code>
	 * &lt;DiversionRoute name="Umleitungsstrecke" description="U1"&gt;
	 * <br>
	 * &lt;Name name="Name"&gt;U1 (A7, Langenau) Richtung Würzburg&lt;/Name&gt;
	 * <br> 
	 * &lt;Description name="Beschreibung"&lt;/Description&gt;
	 * </code>
	 * </p>
	 * <p>
	 * 'Description'-Element ist leer, 'description'-Attribut an
	 * 'DiversionRoute' ist gefüllt!
	 * </p>
	 * <p>
	 * Wir integrieren hier deshalb auch die Beschreibung aus dem Wert des
	 * 'description'-Attributes.
	 * </p>
	 */
	private TIC3InfoAttribut elementBeschreibung = new TIC3InfoAttribut("description", null);

	/**
	 * Lokation.
	 * <p>
	 * TIC3 Info XML Element: Location (ROU), Information about a diversion
	 * route specified by one or more location referencing methods such as TIC3
	 * location, geographical position, or TMC location.
	 * </p>
	 */
	private final TIC3Location tic3Location = new TIC3Location("Location", "Lokation");

	/**
	 * Fahrzeugtyp.
	 * <p>
	 * TIC3 Info XML Element: VehicleType (VTY), The type of the vehicle such as
	 * car, truck, or any type.
	 * </p>
	 */
	private static final String DAV_ATTR_FAHRZEUGTYP = "Fahrzeugtyp";
	private final List<TIC3FahrzeugTyp> fahrzeugTypen = new ArrayList<>();

	/**
	 * Reiseziel.
	 * <p>
	 * TIC3 Info XML Element: Destination (DST), The destination for which the
	 * diversion is specified.
	 * </p>
	 */
	private final TIC3Reiseziel reiseziel = new TIC3Reiseziel("Destination", "Reiseziel");

	/**
	 * Ab Fahrzeuggewicht.
	 * <p>
	 * TIC3 Info XML Element: VehicleWeightOver (VWE), The vehicle weight above
	 * which the diversion route is specified.
	 * </p>
	 */
	private final TIC3OptionalerDoubleWert abGewicht = new TIC3OptionalerDoubleWert("VehicleWeightOver",
			"FahrzeuggewichtAb");

	/**
	 * Ab Fahrzeugbreite.
	 * <p>
	 * TIC3 Info XML Element: VehicleWidthOver (VW1), The vehicle width above
	 * which the diversion route is specified.
	 * </p>
	 */
	private final TIC3OptionalerDoubleWert abBreite = new TIC3OptionalerDoubleWert("VehicleWidthOver",
			"FahrzeugbreiteAb");

	/**
	 * Ab Fahrzeughöhe.
	 * <p>
	 * TIC3 Info XML Element: VehicleHeightOver (VHE), The vehicle height above
	 * which the diversion route is specified.
	 * </p>
	 */
	private final TIC3OptionalerDoubleWert abHoehe = new TIC3OptionalerDoubleWert("VehicleHeightOver",
			"FahrzeughöheAb");

	/**
	 * Ab Fahrzeuglänge.
	 * <p>
	 * TIC3 Info XML Element: VehicleLengthOver (VLE), The vehicle length above
	 * which the diversion route is specified.
	 * </p>
	 */
	private final TIC3OptionalerDoubleWert abLaenge = new TIC3OptionalerDoubleWert("VehicleLengthOver",
			"FahrzeugLängeAb");
	/**
	 * 
	 * /** Maximales Fahrzeuggewicht.
	 * <p>
	 * TIC3 Info XML Element: MaximumVehicleWeight (MVW), The maximum weight of
	 * vehicles which can use the diversion route.
	 * </p>
	 */
	private final TIC3OptionalerDoubleWert maxGewicht = new TIC3OptionalerDoubleWert("MaximumVehicleWeight",
			"MaximalesFahrzeugGewicht");

	/**
	 * Maximale Fahrzeugbreite.
	 * <p>
	 * TIC3 Info XML Element: MaximumVehicleWidth (MVI), The maximum width of
	 * vehicles which can use the diversion route.
	 * </p>
	 */
	private final TIC3OptionalerDoubleWert maxBreite = new TIC3OptionalerDoubleWert("MaximumVehicleWidth",
			"MaximaleFahrzeugBreite");

	/**
	 * Maximale Fahrzeughöhe.
	 * <p>
	 * TIC3 Info XML Element: MaximumVehicleHeight (MVH), The maximum height of
	 * vehicles which can use the diversion route.
	 * </p>
	 */
	private final TIC3OptionalerDoubleWert maxHoehe = new TIC3OptionalerDoubleWert("MaximumVehicleHeight",
			"MaximaleFahrzeugHöhe");

	/**
	 * Maximale Fahrzeuglänge.
	 * <p>
	 * TIC3 Info XML Element: MaximumVehicleLength (MVL), The maximum length of
	 * vehicles which can use the diversion route.
	 * </p>
	 */
	private final TIC3OptionalerDoubleWert maxLaenge = new TIC3OptionalerDoubleWert("MaximumVehicleLength",
			"MaximaleFahrzeugLänge");

	/**
	 * Enthält permanente Umleitungen (Bedarfsumleitung).
	 * <p>
	 * TIC3 Info XML Element: PermanentDiversionRoute (PDR), Indicates whether
	 * or not this route was created using one or more permanent diversion
	 * routes.
	 * </p>
	 */
	private final TIC3Boolean permanenteRouten = new TIC3Boolean("PermanentDiversionRoute", "Bedarfsumleitung");

	/**
	 * Erzeugt ein neues Objekt.
	 *
	 * @param xmlName
	 *            Name des XML-ELementes
	 * @param davName
	 *            Name des DAV-Attributes
	 */
	public TIC3Umleitung(final String xmlName, final String davName) {
		super(xmlName, davName);
		addAttribut(elementBeschreibung);
		addKindElement(name);
		addKindElement(beschreibung);
		addKindElement(tic3Location);
		addKindElement(reiseziel);
		addKindElement(abBreite);
		addKindElement(abGewicht);
		addKindElement(abHoehe);
		addKindElement(abLaenge);
		addKindElement(maxBreite);
		addKindElement(maxGewicht);
		addKindElement(maxHoehe);
		addKindElement(maxLaenge);
		addKindElement(permanenteRouten);
	}

	/**
	 * Erzeugt ein neues Objekt.
	 */
	public TIC3Umleitung() {
		this(XML_ELEMENT_UMLEITUNG, null);
	}

	@Override
	public void einlesenXML(final Node node) throws TIC3Exception {
		super.einlesenXML(node);

		final NodeList nl = node.getChildNodes();

		for (int i = 0; i < nl.getLength(); i++) {
			final Node n = nl.item(i);

			// Fahrzeugtypen
			if (n.getNodeName().equals(TIC3FahrzeugTyp.XML_ELEMENT_FAHRZEUGTYP)) {
				final TIC3FahrzeugTyp fzt = new TIC3FahrzeugTyp();
				fzt.einlesenXML(n);
				fahrzeugTypen.add(fzt);
			}
		}
	}

	@Override
	public void publizieren(final Data davdata, final LMSParameter parameter) throws TIC3Exception {
		super.publizieren(davdata, parameter);

		// Fahrzeugtypen
		if (fahrzeugTypen.size() > 0) {
			final Data.Array array = davdata.getArray(DAV_ATTR_FAHRZEUGTYP);
			array.setLength(fahrzeugTypen.size());
			for (int i = 0; i < fahrzeugTypen.size(); i++) {
				fahrzeugTypen.get(i).publizieren(array.getItem(i), parameter);
			}
		}
	}

	@Override
	public String toString() {
		return this.getClass().getSimpleName();
	}
}
